@@ -41,6 +41,8 @@
 #include "idbm.h"
 #include "iscsi_settings.h"
 #include "util.h"
+#include "sysdeps.h"
+#include "fw_context.h"
 
 #ifdef SLP_ENABLE
 #include "iscsi-slp-discovery.h"
@@ -50,7 +52,51 @@
 
 static int rediscover = 0;
 
-int discovery_offload_sendtargets(idbm_t *db, int host_no, int do_login,
+static char initiator_name[TARGET_NAME_MAXLEN + 1];
+static char initiator_alias[TARGET_NAME_MAXLEN + 1];
+
+int discovery_fw(struct discovery_rec *drec, struct iface_rec *iface,
+		 struct list_head *rec_list)
+{
+	struct boot_context *bcontext;
+	struct list_head targets;
+	struct node_rec *rec;
+	int rc;
+
+	INIT_LIST_HEAD(&targets);
+	rc = fw_get_targets(&targets);
+	if (rc) {
+		log_error("Could not get list of targets from firmware. "
+			  "(err %d)\n", rc);
+		return rc;
+	}
+	if (list_empty(&targets))
+		return 0;
+	/*
+	 * TODO: Do we want to match the iface MAC/netdev with what is in
+	 * the firmware or could the user want to bind based on what is
+	 * in passed in or in the default ifaces?
+	 */
+
+	list_for_each_entry(bcontext, &targets, list) {
+		rec = fw_create_rec_by_entry(bcontext);
+		if (!rec) {
+			log_error("Could not convert firmware info to "
+				  "node record.\n");
+			rc = ENOMEM;
+			goto free_targets;
+		}
+		rec->disc_type = drec->type;
+
+		list_add_tail(&rec->list, rec_list);
+	}
+
+free_targets:
+	fw_free_targets(&targets);
+	return rc;
+}
+
+int discovery_offload_sendtargets(int host_no, int do_login,
 				  discovery_rec_t *drec)
 {
 	struct sockaddr_storage ss;
@@ -180,12 +226,12 @@ iterate_targets(iscsi_session_t *session, uint32_t ttt)
 	return 1;
 }
 
-static int add_portal(idbm_t *db, discovery_rec_t *drec,
-		      struct list_head *ifaces, node_rec_t *rec,
-		      char *address, char *port, char *tag)
+static int add_portal(struct list_head *rec_list, discovery_rec_t *drec,
+		      char *targetname, char *address, char *port, char *tag)
 {
 	struct sockaddr_storage ss;
 	char host[NI_MAXHOST];
+	struct node_rec *rec;
 
 	/* resolve the address, in case it was a DNS name */
 	if (resolve_address(address, port, &ss)) {
@@ -197,6 +243,16 @@ static int add_portal(idbm_t *db, discovery_rec_t *drec,
 	getnameinfo((struct sockaddr *) &ss, sizeof(ss),
 		    host, sizeof(host), NULL, 0, NI_NUMERICHOST);
 
+	rec = calloc(1, sizeof(*rec));
+	if (!rec)
+		return 0;
+
+	idbm_node_setup_from_conf(rec);
+	rec->disc_type = drec->type;
+	rec->disc_port = drec->port;
+	strcpy(rec->disc_address, drec->address);
+
+	strlcpy(rec->name, targetname, TARGET_NAME_MAXLEN);
 	if (tag && *tag)
 		rec->tpgt = atoi(tag);
 	else
@@ -205,23 +261,19 @@ static int add_portal(idbm_t *db, discovery_rec_t *drec,
 		rec->conn[0].port = atoi(port);
 	else
 		rec->conn[0].port = ISCSI_LISTEN_PORT;
-	strncpy(rec->conn[0].address, address, NI_MAXHOST);
+	strlcpy(rec->conn[0].address, address, NI_MAXHOST);
 
-	if (idbm_add_nodes(db, rec, drec, ifaces))
-		log_error("Could not add record for %s %s,%d,%d\n",
-			  rec->name, address, rec->conn[0].port, rec->tpgt);
+	list_add_tail(&rec->list, rec_list);
 	return 1;
 }
 
 static int
-add_target_record(idbm_t *db, char *name, char *end,
-		  discovery_rec_t *drec, struct list_head *ifaces,
-		  char *default_port)
+add_target_record(char *name, char *end, discovery_rec_t *drec,
+		  struct list_head *rec_list, char *default_port)
 {
 	char *text = NULL;
 	char *nul = name;
 	size_t length;
-	node_rec_t rec;
 
 	/* address = IPv4
 	 * address = [IPv6]
@@ -248,10 +300,6 @@ add_target_record(idbm_t *db, char *name, char *end,
 		log_error("TargetName %s too long, ignoring", name);
 		return 0;
 	}
-
-	idbm_node_setup_from_conf(db, &rec);
-	strncpy(rec.name, name, TARGET_NAME_MAXLEN);
-
 	text = name + length;
 
 	/* skip NULs after the name */
@@ -264,7 +312,7 @@ add_target_record(idbm_t *db, char *name, char *end,
 			log_error("no default address known for target %s",
 				  name);
 			return 0;
-		} else if (!add_portal(db, drec, ifaces, &rec, drec->address,
+		} else if (!add_portal(rec_list, drec, name, drec->address,
 				       default_port, NULL)) {
 			log_error("failed to add default portal, ignoring "
 				  "target %s", name);
@@ -302,7 +350,7 @@ add_target_record(idbm_t *db, char *name, char *end,
 					*temp = '\0';
 			}
 
-			if (!add_portal(db, drec, ifaces, &rec, address, port,
+			if (!add_portal(rec_list, drec, name, address, port,
 					tag)) {
 				log_error("failed to add default portal, "
 					 "ignoring target %s", name);
@@ -318,9 +366,9 @@ add_target_record(idbm_t *db, char *name, char *end,
 }
 
 static int
-process_sendtargets_response(idbm_t *db, struct string_buffer *sendtargets,
+process_sendtargets_response(struct string_buffer *sendtargets,
 			     int final, discovery_rec_t *drec,
-			     struct list_head *ifaces,
+			     struct list_head *rec_list,
 			     char *default_port)
 {
 	char *start = buffer_data(sendtargets);
@@ -371,8 +419,8 @@ process_sendtargets_response(idbm_t *db, struct string_buffer *sendtargets,
 				 * the end of. don't bother passing the
 				 * "TargetName=" prefix.
 				 */
-				if (!add_target_record(db, record + 11, text,
-							drec, ifaces,
+				if (!add_target_record(record + 11, text,
+							drec, rec_list,
 							default_port)) {
 					log_error(
 					       "failed to add target record");
@@ -400,8 +448,8 @@ process_sendtargets_response(idbm_t *db, struct string_buffer *sendtargets,
 				 "processing final sendtargets record %p, "
 				 "line %s",
 				 record, record);
-			if (add_target_record (db, record + 11, text,
-					       drec, ifaces, default_port)) {
+			if (add_target_record (record + 11, text,
+					       drec, rec_list, default_port)) {
 				num_targets++;
 				record = NULL;
 				truncate_buffer(sendtargets, 0);
@@ -502,40 +550,43 @@ msecs_until(struct timeval *timer)
 	return msecs;
 }
 
-static int
-soonest_msecs(struct timeval *t1, struct timeval *t2, struct timeval *t3)
+static int request_initiator_name(void)
 {
-	int m1 = msecs_until(t1);
-	int m2 = msecs_until(t2);
-	int m3 = msecs_until(t3);
+	int rc;
+	iscsiadm_req_t req;
+	iscsiadm_rsp_t rsp;
 
-	/* infinity is -1, handle it specically */
-	if ((m1 == -1) && (m2 == -1))
-		return m3;
+	memset(initiator_name, 0, sizeof(initiator_name));
+	initiator_name[0] = '\0';
+	memset(initiator_alias, 0, sizeof(initiator_alias));
+	initiator_alias[0] = '\0';
 
-	if ((m1 == -1) && (m3 == -1))
-		return m2;
+	memset(&req, 0, sizeof(req));
+	req.command = MGMT_IPC_CONFIG_INAME;
 
-	if ((m2 == -1) && (m3 == -1))
-		return m1;
+	rc = do_iscsid(&req, &rsp);
+	if (rc)
+		return EIO;
 
-	if (m1 == -1)
-		return (m2 < m3) ? m2 : m3;
+	if (rsp.u.config.var[0] != '\0')
+		strcpy(initiator_name, rsp.u.config.var);
 
-	if (m2 == -1)
-		return (m1 < m3) ? m1 : m3;
+	memset(&req, 0, sizeof(req));
+	req.command = MGMT_IPC_CONFIG_IALIAS;
 
-	if (m3 == -1)
-		return (m1 < m2) ? m1 : m2;
+	rc = do_iscsid(&req, &rsp);
+	if (rc)
+		/* alias is optional so return ok */
+		return 0;
 
-	if (m1 < m2)
-		return (m1 < m3) ? m1 : m3;
-	else
-		return (m2 < m3) ? m2 : m3;
+	if (rsp.u.config.var[0] != '\0')
+		strcpy(initiator_alias, rsp.u.config.var);
+	return 0;
 }
 
 static iscsi_session_t *
-init_new_session(struct iscsi_sendtargets_config *config)
+init_new_session(struct iscsi_sendtargets_config *config,
+		 struct iface_rec *iface)
 {
 	iscsi_session_t *session;
 
@@ -548,7 +599,6 @@ init_new_session(struct iscsi_sendtargets_config *config)
 	session->conn[0].login_timeout = config->conn_timeo.login_timeout;
 	session->conn[0].auth_timeout = config->conn_timeo.auth_timeout;
 	session->conn[0].active_timeout = config->conn_timeo.active_timeout;
-	session->conn[0].idle_timeout = config->conn_timeo.idle_timeout;
 	session->conn[0].hdrdgst_en = ISCSI_DIGEST_NONE;
 	session->conn[0].datadgst_en = ISCSI_DIGEST_NONE;
 
@@ -576,13 +626,26 @@ init_new_session(struct iscsi_sendtargets_config *config)
 	session->isid[4] = 0;
 	session->isid[5] = 0;
 
-	/* initialize the session */
+	request_initiator_name();
+
+	if (iface && strlen(iface->iname)) {
+		strcpy(initiator_name, iface->iname);
+		/* MNC TODO add iface alias */
+	} else {
+		if (initiator_name[0] == '\0') {
+			log_error("Cannot perform discovery. Initiatorname "
+				  "required.");
+			free(session);
+			return NULL;
+		}
+	}
+
 	session->initiator_name = initiator_name;
 	session->initiator_alias = initiator_alias;
 	session->portal_group_tag = PORTAL_GROUP_TAG_UNKNOWN;
 	session->type = ISCSI_SESSION_TYPE_DISCOVERY;
 done:
-	return(session);
+	return session;
 }
 
 
@@ -623,14 +686,14 @@ setup_authentication(iscsi_session_t *session,
 	}
 
 	/* copy in whatever credentials we have */
-	strncpy(session->username, config->auth.username,
+	strlcpy(session->username, config->auth.username,
 		sizeof (session->username));
 	session->username[sizeof (session->username) - 1] = '\0';
 	if ((session->password_length = config->auth.password_length))
 		memcpy(session->password, config->auth.password,
 		       session->password_length);
 
-	strncpy(session->username_in, config->auth.username_in,
+	strlcpy(session->username_in, config->auth.username_in,
 		sizeof (session->username_in));
 	session->username_in[sizeof (session->username_in) - 1] = '\0';
 	if ((session->password_in_length =
@@ -672,15 +735,14 @@ setup_authentication(iscsi_session_t *session,
 }
 
 static int
-process_recvd_pdu(idbm_t *db, struct iscsi_hdr *pdu,
+process_recvd_pdu(struct iscsi_hdr *pdu,
 		  discovery_rec_t *drec,
-		  struct list_head *ifaces,
+		  struct list_head *rec_list,
 		  iscsi_session_t *session,
 		  struct string_buffer *sendtargets,
 		  char *default_port,
 		  int *active,
 		  int *valid_text,
-		  struct timeval *async_timer,
 		  char *data)
 {
 	int rc=0;
@@ -693,6 +755,7 @@ process_recvd_pdu(idbm_t *db, struct iscsi_hdr *pdu,
 			int final =
 				(text_response->flags & ISCSI_FLAG_CMD_FINAL) ||
 				(text_response-> ttt == ISCSI_RESERVED_TAG);
+			size_t curr_data_length;
 
 			log_debug(4, "discovery session to %s:%d received text"
 				 " response, %d data bytes, ttt 0x%x, "
@@ -706,24 +769,17 @@ process_recvd_pdu(idbm_t *db, struct iscsi_hdr *pdu,
 			/* mark how much more data in the sendtargets
 			 * buffer is now valid
 			 */
-			enlarge_data (sendtargets, dlength);
+			curr_data_length = data_length(sendtargets);
+			enlarge_data(sendtargets, dlength);
+			memcpy(buffer_data(sendtargets) + curr_data_length,
+			       data, dlength);
 
-			/*
-			 * we got a response so clear out the current
-			 * db values
-			 *
-			 * TODO: should we make whether rm the current
-			 * values configurable (maybe a --clear option)
-			 */
-			if (!*valid_text)
-				idbm_new_discovery(db, drec);
 			*valid_text = 1;
-
 			/* process as much as we can right now */
-			process_sendtargets_response(db, sendtargets,
+			process_sendtargets_response(sendtargets,
 						     final,
 						     drec,
-						     ifaces,
+						     rec_list,
 						     default_port);
 
 			if (final) {
@@ -814,22 +870,21 @@ done:
 	iscsi_io_disconnect(&session->conn[0]);
 }
 
-int discovery_sendtargets(idbm_t *db, discovery_rec_t *drec,
-			  struct list_head *ifaces)
+int discovery_sendtargets(discovery_rec_t *drec, struct iface_rec *iface,
+			  struct list_head *rec_list)
 {
 	iscsi_session_t *session;
 	struct pollfd pfd;
 	struct iscsi_hdr pdu_buffer;
 	struct iscsi_hdr *pdu = &pdu_buffer;
 	char *data = NULL;
-	char *end_of_data;
 	int active = 0, valid_text = 0;
-	struct timeval connection_timer, async_timer;
+	struct timeval connection_timer;
 	int timeout;
 	int rc;
 	struct string_buffer sendtargets;
 	uint8_t status_class = 0, status_detail = 0;
-	unsigned int login_failures = 0;
+	unsigned int login_failures = 0, data_len;
 	int login_delay = 0;
 	struct sockaddr_storage ss;
 	char host[NI_MAXHOST], serv[NI_MAXSERV], default_port[NI_MAXSERV];
@@ -840,13 +895,13 @@ int discovery_sendtargets(idbm_t *db, discovery_rec_t *drec,
 		 drec->address, drec->port);
 	memset(&pdu_buffer, 0, sizeof (pdu_buffer));
 	clear_timer(&connection_timer);
-	clear_timer(&async_timer);
 
 	/* allocate a new session, and initialize default values */
-	session = init_new_session(config);
+	session = init_new_session(config, iface);
 	if (session == NULL) {
-		log_error("discovery process to %s:%d failed to "
-			  "allocate a session", drec->address, drec->port);
+		log_error("Discovery process to %s:%d failed to "
+			  "create a discovery session.",
+			  drec->address, drec->port);
 		return 1;
 	}
 
@@ -856,13 +911,15 @@ int discovery_sendtargets(idbm_t *db, discovery_rec_t *drec,
 		 session->isid[1], session->isid[2], session->isid[3],
 		 session->isid[4], session->isid[5]);
 
-	/* allocate data buffers for SendTargets data and discovery pipe info */
-	init_string_buffer(&sendtargets,
-			  session->conn[0].max_recv_dlength);
-	if (!sendtargets.buffer) {
+	/* allocate data buffers for SendTargets data */
+	data = malloc(session->conn[0].max_recv_dlength);
+	if (!data) {
 		rc = 1;
 		goto free_session;
 	}
+	data_len = session->conn[0].max_recv_dlength;
+
+	init_string_buffer(&sendtargets, 0);
 
 	sprintf(default_port, "%d", drec->port);
 	/* resolve the DiscoveryAddress to an IP address */
@@ -872,11 +929,9 @@ int discovery_sendtargets(idbm_t *db, discovery_rec_t *drec,
 		goto free_sendtargets;
 	}
 
-	log_debug(4, "discovery timeouts: login %d, reopen_cnt %d, auth %d, "
-		 "active %d, idle %d, ping %d",
+	log_debug(4, "discovery timeouts: login %d, reopen_cnt %d, auth %d.",
 		 session->conn[0].login_timeout, session->reopen_cnt,
-		 session->conn[0].auth_timeout, session->conn[0].active_timeout,
-		 session->conn[0].idle_timeout, session->conn[0].ping_timeout);
+		 session->conn[0].auth_timeout);
 
 	/* setup authentication variables for the session*/
 	rc = setup_authentication(session, drec, config);
@@ -958,8 +1013,9 @@ redirect_reconnect:
 
 	status_class = 0;
 	status_detail = 0;
-	rc = iscsi_login(session, 0, buffer_data(&sendtargets),
-			 unused_length(&sendtargets),
+
+	memset(data, 0, data_len);
+	rc = iscsi_login(session, 0, data, data_len,
 			 &status_class, &status_detail);
 
 	switch (rc) {
@@ -968,7 +1024,6 @@ redirect_reconnect:
 		break;
 
 	case LOGIN_IO_ERROR:
-	case LOGIN_WRONG_PORTAL_GROUP:
 	case LOGIN_REDIRECTION_FAILED:
 		/* try again */
 		log_warning("retrying discovery login to %s", host);
@@ -1047,13 +1102,9 @@ redirect_reconnect:
 		goto reconnect;
 	}
 
-      rediscover:
 	/* reinitialize */
 	truncate_buffer(&sendtargets, 0);
 
-	/* we're going to do a discovery regardless */
-	clear_timer(&async_timer);
-
 	/* ask for targets */
 	if (!request_targets(session)) {
 		goto reconnect;
@@ -1068,40 +1119,8 @@ redirect_reconnect:
 	pfd.fd = session->conn[0].socket_fd;
 	pfd.events = POLLIN | POLLPRI;
 
-	/* check timers before blocking */
-	if (timer_expired(&connection_timer)) {
-		log_warning("discovery session to %s:%d session "
-			    "logout, connection timer expired",
-			    drec->address, drec->port);
-			    iscsi_logout_and_disconnect(session);
-		rc = 1;
-		goto free_sendtargets;
-	}
-
-	if (active) {
-		/* ignore the async timer, we're in the middle
-		 * of a discovery
-		 */
-		timeout = msecs_until(&connection_timer);
-	} else {
-		/* to avoid doing LUN probing repeatedly, try to merge
-		 * multiple Async PDUs into one rediscovery by
-		 * deferring discovery until a timeout expires.
-		 */
-		if (timer_expired(&async_timer)) {
-			log_debug(4,
-				 "discovery session to %s:%d async "
-				 "timer expired, rediscovering",
-				 drec->address, drec->port);
-			clear_timer(&async_timer);
-			goto rediscover;
-		} else
-			timeout =
-				soonest_msecs(NULL,
-					  &connection_timer,
-					  &async_timer);
-	}
-
+repoll:
+	timeout = msecs_until(&connection_timer);
 	/* block until we receive a PDU, a TCP FIN, a TCP RST,
 	 * or a timeout
 	 */
@@ -1118,50 +1137,24 @@ redirect_reconnect:
 		 "discovery process to %s:%d returned from poll, rc %d",
 		 drec->address, drec->port, rc);
 
+	if (timer_expired(&connection_timer)) {
+		log_warning("discovery session to %s:%d session "
+			    "logout, connection timer expired",
+			    drec->address, drec->port);
+			    iscsi_logout_and_disconnect(session);
+		rc = 1;
+		goto free_sendtargets;
+	}
+
 	if (rc > 0) {
 		if (pfd.revents & (POLLIN | POLLPRI)) {
-			/* put any PDU data into the
-			 * sendtargets buffer for now
-			 */
-			data = buffer_data(&sendtargets) +
-			    data_length(&sendtargets);
-			end_of_data =
-			    data + unused_length(&sendtargets);
 			timeout = msecs_until(&connection_timer);
 
-			if (iscsi_io_recv_pdu(&session->conn[0],
-			     pdu, ISCSI_DIGEST_NONE, data,
-			     end_of_data - data, ISCSI_DIGEST_NONE,
-			     timeout)) {
-				/*
-				 * process iSCSI PDU received
-				 */
-				rc = process_recvd_pdu(
-					db,
-					pdu,
-					drec,
-					ifaces,
-					session,
-					&sendtargets,
-					default_port,
-					&active,
-					&valid_text,
-					&async_timer,
-					data);
-				if (rc == DISCOVERY_NEED_RECONNECT)
-					goto reconnect;
-
-				/* reset timers after receiving a PDU */
-				if (active)
-					set_timer(&connection_timer,
-					       session->conn[0].active_timeout);
-				else
-					/*
-					 * 3 minutes to try
-					 * to go long-lived
-					 */
-					set_timer(&connection_timer, 3 * 60);
-			} else {
+			memset(data, 0, data_len);
+			if (!iscsi_io_recv_pdu(&session->conn[0],
+					       pdu, ISCSI_DIGEST_NONE, data,
+			     		       data_len, ISCSI_DIGEST_NONE,
+					       timeout)) {
 				log_debug(1, "discovery session to "
 					  "%s:%d failed to recv a PDU "
 					  "response, terminating",
@@ -1171,7 +1164,25 @@ redirect_reconnect:
 				rc = 1;
 				goto free_sendtargets;
 			}
+
+			/*
+			 * process iSCSI PDU received
+			 */
+			rc = process_recvd_pdu(pdu, drec, rec_list,
+					       session, &sendtargets,
+					       default_port,
+					       &active, &valid_text, data);
+			if (rc == DISCOVERY_NEED_RECONNECT)
+				goto reconnect;
+
+			/* reset timers after receiving a PDU */
+			if (active) {
+				set_timer(&connection_timer,
+				       session->conn[0].active_timeout);
+				goto repoll;
+			}
 		}
+
 		if (pfd.revents & POLLHUP) {
 			log_warning("discovery session to %s:%d "
 				    "terminating after hangup",
@@ -1213,6 +1224,7 @@ redirect_reconnect:
 
 free_sendtargets:
 	free_string_buffer(&sendtargets);
+	free(data);
 free_session:
 	free(session);
 	return rc;
