@@ -33,6 +33,8 @@ enum {
 	ISCSI_STRING,
 };
 
+struct iscsi_conn;
+
 /**
  * struct iscsi_ipc - Open-iSCSI Interface for Kernel IPC
  *
@@ -53,13 +55,15 @@ struct iscsi_ipc {
 	int (*sendtargets) (uint64_t transport_handle, uint32_t host_no,
 			    struct sockaddr *addr);
 
-	int (*create_session) (uint64_t transport_handle,
+	int (*create_session) (uint64_t transport_handle, uint64_t ep_handle,
 			       uint32_t initial_cmdsn, uint16_t cmds_max,
 			       uint16_t qdepth, uint32_t *out_sid,
-			       uint32_t *out_hostno);
+			       uint32_t *hostno);
 
 	int (*destroy_session) (uint64_t transport_handle, uint32_t sid);
 
+	int (*unbind_session) (uint64_t transport_handle, uint32_t sid);
+
 	int (*create_conn) (uint64_t transport_handle,
 			    uint32_t sid, uint32_t cid, uint32_t *out_cid);
 
@@ -102,11 +106,9 @@ struct iscsi_ipc {
 
 	int (*writev) (enum iscsi_uevent_e type, struct iovec *iovp, int count);
 
-	int (*recv_pdu_begin) (uint64_t transport_handle, uintptr_t recv_handle,
-			       uintptr_t *pdu_handle, int *pdu_size);
+	int (*recv_pdu_begin) (struct iscsi_conn *conn);
 
-	int (*recv_pdu_end) (uint64_t transport_handle, uintptr_t conn_handle,
-			     uintptr_t pdu_handle);
+	int (*recv_pdu_end) (struct iscsi_conn *conn);
 };
 
 #endif /* ISCSI_IPC_H */
