@@ -19,7 +19,6 @@
  *
  * See the file COPYING included with this distribution for more details.
  */
-#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -28,16 +27,12 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
-#include <dirent.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/mman.h>
-#include <sys/poll.h>
 #include <sys/utsname.h>
-#include <sys/signal.h>
 
 #include "iscsid.h"
 #include "mgmt_ipc.h"
+#include "event_poll.h"
 #include "iscsi_ipc.h"
 #include "log.h"
 #include "util.h"
@@ -46,7 +41,9 @@
 #include "idbm.h"
 #include "version.h"
 #include "iscsi_sysfs.h"
-#include "iscsi_settings.h"
+#include "iface.h"
+#include "session_info.h"
+#include "sysdeps.h"
 
 /* global config info */
 struct iscsi_daemon_config daemon_config;
@@ -56,8 +53,6 @@ static char program_name[] = "iscsid";
 int control_fd, mgmt_ipc_fd;
 static pid_t log_pid;
 
-extern char sysfs_file[];
-
 static struct option const long_options[] = {
 	{"config", required_argument, NULL, 'c'},
 	{"initiatorname", required_argument, NULL, 'i'},
@@ -95,24 +90,23 @@ Open-iSCSI initiator daemon.\n\
 }
 
 static void
-setup_rec_from_negotiated_values(idbm_t *db, node_rec_t *rec,
-				struct session_info *info)
+setup_rec_from_negotiated_values(node_rec_t *rec, struct session_info *info)
 {
 	struct iscsi_session_operational_config session_conf;
 	struct iscsi_conn_operational_config conn_conf;
 	struct iscsi_auth_config auth_conf;
 
-	idbm_node_setup_from_conf(db, rec);
-	strncpy(rec->name, info->targetname, TARGET_NAME_MAXLEN);
+	idbm_node_setup_from_conf(rec);
+	strlcpy(rec->name, info->targetname, TARGET_NAME_MAXLEN);
 	rec->conn[0].port = info->persistent_port;
-	strncpy(rec->conn[0].address, info->persistent_address, NI_MAXHOST);
+	strlcpy(rec->conn[0].address, info->persistent_address, NI_MAXHOST);
 	memcpy(&rec->iface, &info->iface, sizeof(struct iface_rec));
 	rec->tpgt = info->tpgt;
 	iface_copy(&rec->iface, &info->iface);
 
-	get_negotiated_session_conf(info->sid, &session_conf);
-	get_negotiated_conn_conf(info->sid, &conn_conf);
-	get_auth_conf(info->sid, &auth_conf);
+	iscsi_sysfs_get_negotiated_session_conf(info->sid, &session_conf);
+	iscsi_sysfs_get_negotiated_conn_conf(info->sid, &conn_conf);
+	iscsi_sysfs_get_auth_conf(info->sid, &auth_conf);
 
 	if (strlen(auth_conf.username))
 		strcpy(rec->session.auth.username, auth_conf.username);
@@ -183,17 +177,17 @@ setup_rec_from_negotiated_values(idbm_t *db, node_rec_t *rec,
 
 static int sync_session(void *data, struct session_info *info)
 {
-	idbm_t *db = data;
-	node_rec_t rec;
+	node_rec_t rec, sysfsrec;
 	iscsiadm_req_t req;
 	iscsiadm_rsp_t rsp;
 	struct iscsi_transport *t;
+	int rc, retries = 0;
 
 	log_debug(7, "sync session [%d][%s,%s.%d][%s]\n", info->sid,
 		  info->targetname, info->persistent_address,
 		  info->port, info->iface.hwaddress);
 
-	t = get_transport_by_sid(info->sid);
+	t = iscsi_sysfs_get_transport_by_sid(info->sid);
 	if (!t)
 		return 0;
 
@@ -205,69 +199,122 @@ static int sync_session(void *data, struct session_info *info)
 		uint32_t host_no;
 		int err;
 
-		host_no = get_host_no_from_sid(info->sid, &err);
+		host_no = iscsi_sysfs_get_host_no_from_sid(info->sid, &err);
 		if (err) {
 			log_error("Could not get host no from sid %u. Can not "
 				  "sync session. Error %d", info->sid, err);
 			return 0;
 		}
-		scan_host(host_no, 0);
+		iscsi_sysfs_scan_host(host_no, 0);
 		return 0;
 	}
 
 	memset(&rec, 0, sizeof(node_rec_t));
-	if (iface_get_by_bind_info(db, &info->iface, &rec.iface)) {
-		log_warning("Could not read data from db. Using default and "
-			    "currently negotiated values\n");
-		setup_rec_from_negotiated_values(db, &rec, info);
-	} else if (idbm_rec_read(db, &rec, info->targetname, info->tpgt,
+	if (idbm_rec_read(&rec, info->targetname, info->tpgt,
 			  info->persistent_address, info->persistent_port,
-			  &rec.iface)) {
+			  &info->iface)) {
 		log_warning("Could not read data from db. Using default and "
 			    "currently negotiated values\n");
-		setup_rec_from_negotiated_values(db, &rec, info);
+		setup_rec_from_negotiated_values(&rec, info);
+	} else {
+		/*
+		 * we have a valid record and iface so lets merge
+		 * the values from them and sysfs to try and get
+		 * the most uptodate values.
+		 *
+		 * Currenlty that means we will use the CHAP, target and
+		 * and portal values from sysfs and use timer, queue depth,
+		 * and segment length values from the record. In the future
+		 * when boot supports iface binding we will want to use
+		 * those values from sysfs.
+		 */
+		memset(&sysfsrec, 0, sizeof(node_rec_t));
+		setup_rec_from_negotiated_values(&sysfsrec, info);
+		/*
+		 * target and portal values have to be the same or
+		 * we would not have found the record, so just copy
+		 * CHAP.
+		 */
+		memcpy(&rec.session.auth, &sysfsrec.session.auth,
+		      sizeof(struct iscsi_auth_config));
 	}
 
 	/* multiple drivers could be connected to the same portal */
 	if (!iscsi_match_session(&rec, info))
 		return -1;
-
+	/*
+	 * We use the initiator name from sysfs because
+	 * the session could have come from our db or ibft or some other
+	 * app.
+	 */
+	strcpy(rec.iface.iname, info->iface.iname);
 	memset(&req, 0, sizeof(req));
 	req.command = MGMT_IPC_SESSION_SYNC;
 	req.u.session.sid = info->sid;
 	memcpy(&req.u.session.rec, &rec, sizeof(node_rec_t));
 
-	do_iscsid(&req, &rsp);
+retry:
+	rc = do_iscsid(&req, &rsp);
+	if (rc == MGMT_IPC_ERR_ISCSID_NOTCONN && retries < 30) {
+		retries++;
+		sleep(1);
+		goto retry;
+	}
 	return 0;
 }
 
-static void sync_sessions(void)
+static char *iscsid_get_config_file(void)
 {
-	idbm_t *db;
-	int nr_found = 0;
-
-	db = idbm_init(daemon_config.config_file);
-	if (!db)
-		return;
-	sysfs_for_each_session(db, &nr_found, sync_session);
-	idbm_terminate(db);
-}
-
-static void catch_signal(int signo)
-{
-	log_warning("caught signal -%d, ignoring...", signo);
+	return daemon_config.config_file;
 }
 
 static void iscsid_exit(void)
 {
-	log_debug(1, "iscsid_exit");
+	isns_exit();
+	ipc->ctldev_close();
+	mgmt_ipc_close(mgmt_ipc_fd);
 	if (daemon_config.initiator_name)
 		free(daemon_config.initiator_name);
 	if (daemon_config.initiator_alias)
 		free(daemon_config.initiator_alias);
 	free_initiator();
-	mgmt_ipc_close(mgmt_ipc_fd);
-	ipc->ctldev_close();
+}
+
+static void iscsid_shutdown(void)
+{
+	log_warning("iscsid shutting down.");
+	if (log_daemon && log_pid >= 0) {
+		log_debug(1, "daemon stopping");
+		log_close(log_pid);
+		fprintf(stderr, "done done\n");
+	}
+	exit(0);
+}
+
+static void catch_signal(int signo)
+{
+	log_debug(1, "%d caught signal -%d...", signo, getpid());
+
+	switch (signo) {
+	case SIGTERM:
+		iscsid_shutdown();
+		break;
+	default:
+		break;
+	}
+}
+
+static void missing_iname_warn(char *initiatorname_file)
+{
+	fprintf(stderr, "Warning: initiatorname file %s doesn't "
+		"exist. If using software iscsi (iscsi_tcp or ib_iser) or "
+		"partial offload (bnx iscsi), you may not be able to log "
+		"into or discovery targets. Please create a file %s that "
+		"contains a sting with the format: InitiatorName="
+		"iqn.yyyy-mm.<reversed domain name>[:identifier].\n\n"
+		"Example: InitiatorName=iqn.2001-04.com.redhat:fc6.\n"
+		"If using hardware iscsi like qla4xxx this message can be "
+		"ignored.\n", initiatorname_file, initiatorname_file);
 }
 
 int main(int argc, char *argv[])
@@ -329,24 +376,21 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	/* make sure we have initiatorname config file first */
-	if (access(initiatorname_file, R_OK)) {
-		fprintf(stderr, "Error: initiatorname file %s doesn't exist.\n",
-			initiatorname_file);
-		fprintf(stderr, "Please create a file %s that contains\n",
-			initiatorname_file);
-		fprintf(stderr, "a sting with the format: InitiatorName="
-			"iqn.yyyy-mm.<reversed domain name>[:identifier].\n");
-		fprintf(stderr, "Example: InitiatorName=iqn.2001-04.com.redhat:"
-			"fc6\n");
-		usage(0);
-	}
-
 	/* initialize logger */
 	log_pid = log_init(program_name, DEFAULT_AREA_SIZE);
 	if (log_pid < 0)
 		exit(1);
-	check_class_version();
+
+	sysfs_init();
+	if (idbm_init(iscsid_get_config_file)) {
+		log_close(log_pid);
+		exit(1);
+	}
+
+	if (iscsi_sysfs_check_class_version()) {
+		log_close(log_pid);
+		exit(1);
+	}
 
 	umask(0177);
 
@@ -356,11 +400,14 @@ int main(int argc, char *argv[])
 	daemon_config.initiator_alias = NULL;
 	if (atexit(iscsid_exit)) {
 		log_error("failed to set exit function\n");
+		log_close(log_pid);
 		exit(1);
 	}
 
-	if ((mgmt_ipc_fd = mgmt_ipc_listen()) < 0)
-		exit(-1);
+	if ((mgmt_ipc_fd = mgmt_ipc_listen()) < 0) {
+		log_close(log_pid);
+		exit(1);
+	}
 
 	if (log_daemon) {
 		char buf[64];
@@ -369,23 +416,28 @@ int main(int argc, char *argv[])
 		fd = open(pid_file, O_WRONLY|O_CREAT, 0644);
 		if (fd < 0) {
 			log_error("Unable to create pid file");
+			log_close(log_pid);
 			exit(1);
 		}
 		pid = fork();
 		if (pid < 0) {
 			log_error("Starting daemon failed");
+			log_close(log_pid);
 			exit(1);
 		} else if (pid) {
 			log_error("iSCSI daemon with pid=%d started!", pid);
 			exit(0);
 		}
 
-		if ((control_fd = ipc->ctldev_open()) < 0)
-			exit(-1);
+		if ((control_fd = ipc->ctldev_open()) < 0) {
+			log_close(log_pid);
+			exit(1);
+		}
 
 		chdir("/");
 		if (lockf(fd, F_TLOCK, 0) < 0) {
 			log_error("Unable to lock pid file");
+			log_close(log_pid);
 			exit(1);
 		}
 		ftruncate(fd, 0);
@@ -394,8 +446,10 @@ int main(int argc, char *argv[])
 
 		daemon_init();
 	} else {
-		if ((control_fd = ipc->ctldev_open()) < 0)
-			exit(-1);
+		if ((control_fd = ipc->ctldev_open()) < 0) {
+			log_close(log_pid);
+			exit(1);
+		}
 	}
 
 	if (uid && setuid(uid) < 0)
@@ -407,17 +461,14 @@ int main(int argc, char *argv[])
 	memset(&daemon_config, 0, sizeof (daemon_config));
 	daemon_config.pid_file = pid_file;
 	daemon_config.config_file = config_file;
-	daemon_config.initiator_name_file = initiatorname_file;
 	daemon_config.initiator_name =
-	    get_iscsi_initiatorname(daemon_config.initiator_name_file);
-	if (daemon_config.initiator_name == NULL) {
-		log_warning("exiting due to configuration error");
-		exit(1);
-	}
+				get_iscsi_initiatorname(initiatorname_file);
+	if (daemon_config.initiator_name == NULL)
+		missing_iname_warn(initiatorname_file);
 
 	/* optional InitiatorAlias */
 	daemon_config.initiator_alias =
-	    get_iscsi_initiatoralias(daemon_config.initiator_name_file);
+				get_iscsi_initiatoralias(initiatorname_file);
 	if (!daemon_config.initiator_alias) {
 		memset(&host_info, 0, sizeof (host_info));
 		if (uname(&host_info) >= 0) {
@@ -426,13 +477,15 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	log_debug(1, "InitiatorName=%s", daemon_config.initiator_name);
+	log_debug(1, "InitiatorName=%s", daemon_config.initiator_name ?
+		 daemon_config.initiator_name : "NOT SET");
 	log_debug(1, "InitiatorAlias=%s", daemon_config.initiator_alias);
 
 	pid = fork();
 	if (pid == 0) {
+		int nr_found = 0;
 		/* child */
-		sync_sessions();
+		iscsi_sysfs_for_each_session(NULL, &nr_found, sync_session);
 		exit(0);
 	} else if (pid < 0) {
 		log_error("Fork failed error %d: existing sessions"
@@ -447,14 +500,16 @@ int main(int argc, char *argv[])
 	/* we don't want our active sessions to be paged out... */
 	if (mlockall(MCL_CURRENT | MCL_FUTURE)) {
 		log_error("failed to mlockall, exiting...");
+		log_close(log_pid);
 		exit(1);
 	}
 
+	increase_max_files();
 	actor_init();
 	isns_fd = isns_init();
 	event_loop(ipc, control_fd, mgmt_ipc_fd, isns_fd);
-	isns_exit();
-
-	log_debug(1, "daemon stopping");
+	iscsid_shutdown();
+	idbm_terminate();
+	sysfs_cleanup();
 	return 0;
 }
