@@ -25,10 +25,10 @@
 #include "log.h"
 #include "util.h"
 #include "iscsi_sysfs.h"
+#include "cxgb3i.h"
 
 struct iscsi_transport_template iscsi_tcp = {
 	.name		= "tcp",
-	.rdma		= 0,
 	.ep_connect	= iscsi_io_tcp_connect,
 	.ep_poll	= iscsi_io_tcp_poll,
 	.ep_disconnect	= iscsi_io_tcp_disconnect,
@@ -42,14 +42,30 @@ struct iscsi_transport_template iscsi_iser = {
 	.ep_disconnect	= ktransport_ep_disconnect,
 };
 
+struct iscsi_transport_template cxgb3i = {
+	.name		= "cxgb3i",
+	.ep_connect	= ktransport_ep_connect,
+	.ep_poll	= ktransport_ep_poll,
+	.ep_disconnect	= ktransport_ep_disconnect,
+	.create_conn	= cxgb3i_create_conn,
+};
+
+struct iscsi_transport_template bnx2i = {
+	.name		= "bnx2i",
+	.ep_connect	= ktransport_ep_connect,
+	.ep_poll	= ktransport_ep_poll,
+	.ep_disconnect	= ktransport_ep_disconnect,
+};
+
 struct iscsi_transport_template qla4xxx = {
 	.name		= "qla4xxx",
-	.rdma		= 0,
 };
 
 static struct iscsi_transport_template *iscsi_transport_templates[] = {
 	&iscsi_tcp,
 	&iscsi_iser,
+	&cxgb3i,
+	&bnx2i,
 	&qla4xxx,
 	NULL
 };
@@ -70,5 +86,5 @@ int set_transport_template(struct iscsi_transport *t)
 	}
 
 	log_error("Could not fund uspace transport for %s\n", t->name);
-	return -ENOSYS;
+	return ENOSYS;
 }
