@@ -26,9 +26,10 @@ struct iscsi_conn;
 struct iscsi_transport_template {
 	const char *name;
 	uint8_t rdma;
-	int (*ep_connect) (iscsi_conn_t *conn, int non_blocking);
-	int (*ep_poll) (iscsi_conn_t *conn, int timeout_ms);
-	void (*ep_disconnect) (iscsi_conn_t *conn);
+	int (*ep_connect) (struct iscsi_conn *conn, int non_blocking);
+	int (*ep_poll) (struct iscsi_conn *conn, int timeout_ms);
+	void (*ep_disconnect) (struct iscsi_conn *conn);
+	void (*create_conn) (struct iscsi_conn *conn);
 };
 
 /* represents data path provider */
