@@ -32,24 +32,16 @@
 #include <errno.h>
 
 #include "fwparam_ibft.h"
+#include "fw_context.h"
 
-char *progname;
+char *progname = "fwparam_ibft";
 int debug;
-char default_file_name[] = "/dev/mem";
-char *filename = default_file_name;
-int boot_selected_only;
+int dev_count;
+char filename[FILENAMESZ];
 
 const char nulls[16]; /* defaults to zero */
 
-/*
- * Prefix strings, for the "prefixN:NAME=value".
- */
-#define NETWORK		"network"
-#define INITIATOR	"iscsi-initiator"
-#define TGT		"target"
-
-
-void
+int
 verify_hdr(char *name, struct ibft_hdr *hdr, int id, int version, int length)
 {
 #define VERIFY_HDR_FIELD(val) \
@@ -58,7 +50,7 @@ verify_hdr(char *name, struct ibft_hdr *hdr, int id, int version, int length)
 			"%s: error, %s structure expected %s %d but" \
 			" got %d\n", \
 			progname, name, #val, hdr->val, val); \
-		exit(1); \
+		return -1; \
 	}
 
 	if (debug > 1)
@@ -70,6 +62,7 @@ verify_hdr(char *name, struct ibft_hdr *hdr, int id, int version, int length)
 	VERIFY_HDR_FIELD(length);
 
 #undef VERIFY_HDR_FIELD
+	return 0;
 }
 
 #define CHECK_HDR(ident, name) \
@@ -314,20 +307,14 @@ dump_tgt_prefix(void *ibft_loc, struct ibft_tgt *tgt, char *prefix)
  * Read in and dump ASCII output for ibft starting at ibft_loc.
  */
 int
-dump_ibft(void *ibft_loc)
+dump_ibft(void *ibft_loc, struct boot_context *context)
 {
 	struct ibft_table_hdr *ibft_hdr = ibft_loc;
 	struct ibft_control *control;
 	struct ibft_initiator *initiator = NULL;
 	struct ibft_nic *nic0 = NULL, *nic1 = NULL;
 	struct ibft_tgt *tgt0 = NULL, *tgt1 = NULL;
-	char sum = 0, *buf = ibft_loc;
-	char prefix[32];
-
-	for (; buf <= (char *) (ibft_loc + ibft_hdr->length);)
-		sum += *buf++;
-	if (sum)
-		fprintf(stderr, "Checksum not zero 0x%x \n", sum);
+	char ipbuf[32];
 
 	control = ibft_loc + sizeof(*ibft_hdr);
 	CHECK_HDR(control, control);
@@ -367,128 +354,122 @@ dump_ibft(void *ibft_loc)
 		CHECK_HDR(tgt1, target);
 	}
 
-	if (boot_selected_only) {
+	strlcpy(context->initiatorname,
+		(char *)ibft_loc+initiator->initiator_name_off,
+		initiator->initiator_name_len + 1);
 
-		snprintf(prefix, sizeof(prefix), "iSCSI_INITIATOR_");
-
-		if (initiator && (initiator->hdr.flags &
-				   INIT_FLAG_FW_SEL_BOOT))
-			dump_initiator_prefix(ibft_loc, initiator, prefix);
-
-		if (nic0 && (nic0->hdr.flags & INIT_FLAG_FW_SEL_BOOT))
-			dump_nic_prefix(ibft_loc, nic0, prefix);
-		else if (nic1 && (nic1->hdr.flags & INIT_FLAG_FW_SEL_BOOT))
-			dump_nic_prefix(ibft_loc, nic1, prefix);
-
-		snprintf(prefix, sizeof(prefix), "iSCSI_TARGET_");
-		if (tgt0 && (tgt0->hdr.flags & INIT_FLAG_FW_SEL_BOOT))
-			dump_tgt_prefix(ibft_loc, tgt0, prefix);
-		else if (tgt1 && (tgt1->hdr.flags & INIT_FLAG_FW_SEL_BOOT))
-			dump_tgt_prefix(ibft_loc, tgt1, prefix);
-
-	} else {
-
-		snprintf(prefix, sizeof(prefix), "%s%d:", INITIATOR, 0);
-		dump_initiator_prefix(ibft_loc, initiator, prefix);
-
-		snprintf(prefix, sizeof(prefix), "%s%d:", NETWORK, 0);
-		dump_nic_prefix(ibft_loc, nic0, prefix);
-		snprintf(prefix, sizeof(prefix), "%s%d:", TGT, 0);
-		dump_tgt_prefix(ibft_loc, tgt0, prefix);
-
-		snprintf(prefix, sizeof(prefix), "%s%d:", NETWORK, 1);
-		dump_nic_prefix(ibft_loc, nic1, prefix);
-		snprintf(prefix, sizeof(prefix), "%s%d:", TGT, 1);
-		dump_tgt_prefix(ibft_loc, tgt1, prefix);
+	if (tgt0 && (tgt0->hdr.flags & INIT_FLAG_FW_SEL_BOOT)) {
+		strlcpy((char *)context->targetname,
+			(char *)(ibft_loc+tgt0->tgt_name_off),
+			tgt0->tgt_name_len);
+		format_ipaddr(ipbuf, sizeof(ipbuf),
+			      tgt0->ip_addr);
+		strlcpy((char *)context->target_ipaddr, ipbuf,
+			sizeof(ipbuf));
+		context->target_port = tgt0->port;
+		strlcpy(context->chap_name,
+			(char *)(ibft_loc + tgt0->chap_name_off),
+			tgt0->chap_name_len);
+		strlcpy(context->chap_password,
+			(char*)(ibft_loc + tgt0->chap_secret_off),
+			tgt0->chap_secret_len);
+		strlcpy(context->chap_name_in,
+			(char *)(ibft_loc + tgt0->rev_chap_name_off),
+			tgt0->rev_chap_name_len);
+		strlcpy(context->chap_password_in,
+			(char *)(ibft_loc + tgt0->rev_chap_secret_off),
+			tgt0->rev_chap_secret_len);
+	} else if (tgt1 &&
+		   (tgt1->hdr.flags & INIT_FLAG_FW_SEL_BOOT)) {
+		strlcpy((char *)context->targetname,
+			(char *)(ibft_loc+tgt1->tgt_name_off),
+			tgt1->tgt_name_len);
+		format_ipaddr(ipbuf, sizeof(ipbuf),
+			      tgt1->ip_addr);
+		strlcpy((char *)context->target_ipaddr,ipbuf,
+			sizeof(ipbuf));
+		context->target_port = tgt1->port;
+		strlcpy(context->chap_name,
+			(char *)(ibft_loc + tgt1->chap_name_off),
+			tgt1->chap_name_len);
+		strlcpy(context->chap_password,
+			(char*)(ibft_loc + tgt1->chap_secret_off),
+			tgt1->chap_secret_len);
+		strlcpy(context->chap_name_in,
+			(char *)(ibft_loc + tgt1->rev_chap_name_off),
+			tgt1->rev_chap_name_len);
+		strlcpy(context->chap_password_in,
+			(char *)(ibft_loc + tgt1->rev_chap_secret_off),
+			tgt1->rev_chap_secret_len);
 	}
 
 	return 0;
 }
 
-/*
- * return the address of the location of string in filebuf, search up to
- * max bytes of *filebuf, if not found returns NULL.
- */
-char *
-search_file(char *filebuf, char *string, int len, int max)
+char *search_ibft(unsigned char *start, int length)
 {
-	char *cur = filebuf;
-	char *end = filebuf + max;
-	int i = 0;
+	unsigned char *cur_ptr;
+	struct ibft_table_hdr *ibft_hdr;
+	unsigned char check_sum;
+	uint32_t i;
 
-	if (debug > 1) {
-		fprintf(stderr,
-			"%s: cur 0x%p, end 0x%p, string '%.4s', len %d\n",
-			__FUNCTION__, cur, end, string, len);
-	}
-	while ((cur < end) && memcmp(cur, string, len)) {
-		if (debug > 2) {
-			fprintf(stderr, "i %d, cur 0x%p: 0x%x ('%c')\n",
-				i, cur, cur[0], cur[0]);
-			i++;
+	cur_ptr = (unsigned char *)start;
+	for (cur_ptr = (unsigned char *)start;
+	     cur_ptr < (start + length);
+	     cur_ptr++) {
+		if (memcmp(cur_ptr, iBFTSTR,strlen(iBFTSTR)))
+			continue;
+
+		ibft_hdr = (struct ibft_table_hdr *)cur_ptr;
+		/* Make sure it's correct version. */
+		if (ibft_hdr->revision != iBFT_REV)
+			continue;
+
+		/* Make sure that length is valid. */
+		if ((cur_ptr + ibft_hdr->length) <= (start + length)) {
+			/* Let verify the checksum */
+			for (i = 0, check_sum = 0; i < ibft_hdr->length; i++)
+				check_sum += cur_ptr[i];
+
+			if (check_sum == 0)
+				return (char *)cur_ptr;
 		}
-		cur++;
 	}
-	if (cur < end)
-		return cur;
-	else
-		return NULL;
+	return NULL;
 }
 
 int
-main (int argc, char **argv)
+fwparam_ibft(struct boot_context *context, const char *filepath)
 {
-	int fd, option, ret;
+	int fd, ret;
 	char *filebuf, *ibft_loc;
 	int start = 512 * 1024; /* 512k */
 	int end_search = (1024 * 1024) - start; /* 512k */
+	struct stat buf;
 
-	progname = argv[0];
+	if (filepath)
+		strlcpy(filename, filepath, FILENAMESZ);
+	else
+		strlcpy(filename, X86_DEFAULT_FILENAME, FILENAMESZ);
 
-	while (1) {
-		option = getopt(argc, argv, "f:m:s:e:vhb");
-		if (option == -1)
-			break;
-		switch (option) {
-		case 'b':
-			boot_selected_only = 1;
-			break;
-		case 'e':
-			end_search = strtoul(optarg, NULL, 0);
-			break;
-		case 'f':
-			filename = optarg;
-			break;
-		case 's':
-			start = strtoul(optarg, NULL, 0);
-			break;
-		case 'v':
-			debug++;
-			break;
-		default:
-			fprintf(stderr, "Unknown or bad option '%c'\n", option);
-		case 'h':
-			printf("Usage: %s OPTIONS\n"
-			       "-b print only fw boot selected sections\n"
-			       "-f file_to_search (default /dev/mem)\n"
-			       "-s offset to start search\n"
-			       "-e length of search\n"
-			       "-v verbose\n",
-			       progname);
-			exit(1);
-		}
-	}
-
-	if (debug)
-		fprintf(stderr, "file: %s; start %d, end_search %d, debug %d\n",
-			filename, start, end_search, debug);
 	fd = open(filename, O_RDONLY);
 	if (fd < 0) {
 		fprintf(stderr, "Could not open %s: %s (%d)\n",
 			filename, strerror(errno), errno);
-		exit(1);
+		return -1;
 	}
 
+	/* Find the size. */
+	if (stat(filename, &buf)!=0) {
+		fprintf(stderr, "Could not stat file %s: %s (%d)\n",
+			filename, strerror(errno), errno);
+		return -1;
+	}
+	/* And if not zero use that size */
+	if (buf.st_size > 0) {
+		start = 0;
+		end_search=buf.st_size;
+	}
 	/*
 	 * XXX Possibly warn and exit if start > filesize(fd), or if start +
 	 * end_search > filesize(fd). Else, we will get a bus error for
@@ -499,18 +480,19 @@ main (int argc, char **argv)
 	if (filebuf == MAP_FAILED) {
 		fprintf(stderr, "Could not mmap %s: %s (%d)\n",
 			filename, strerror(errno), errno);
-		exit(1);
+		ret = -1;
+		goto done;
 	}
 
-	ibft_loc = search_file(filebuf, iBFTSTR, strlen(iBFTSTR), end_search);
-	if (ibft_loc) {
-		if (dump_ibft(ibft_loc))
-			ret = 0;
-		else
-			ret = 1;
-	} else
-		ret = 1;
+	ibft_loc = search_ibft((unsigned char *)filebuf, end_search);
+	if (ibft_loc)
+		ret = dump_ibft(ibft_loc, context);
+	else {
+		printf("Could not find iBFT.\n");
+		ret = -1;
+	}
 	munmap(filebuf, end_search);
+done:
 	close(fd);
-	exit(ret);
+	return ret;
 }
